X-Git-Url: https://git.rapsys.eu/packbundle/blobdiff_plain/a3d21dc7b3d01a8e02af3e88cd263adc259f909d..fedff27fa8b2f164d11e5fa218118f6bc10a57e4:/Parser/TokenParser.php?ds=inline

diff --git a/Parser/TokenParser.php b/Parser/TokenParser.php
index 7749ad2..0ca3e30 100644
--- a/Parser/TokenParser.php
+++ b/Parser/TokenParser.php
@@ -1,4 +1,13 @@
-<?php
+<?php declare(strict_types=1);
+
+/*
+ * This file is part of the Rapsys PackBundle package.
+ *
+ * (c) Raphaël Gertz <symfony@rapsys.eu>
+ *
+ * For the full copyright and license information, please view the LICENSE
+ * file that was distributed with this source code.
+ */
 
 namespace Rapsys\PackBundle\Parser;
 
@@ -16,50 +25,39 @@ use Twig\Source;
 use Twig\Token;
 use Twig\TokenParser\AbstractTokenParser;
 
+use Rapsys\PackBundle\RapsysPackBundle;
+
+/**
+ * {@inheritdoc}
+ */
 class TokenParser extends AbstractTokenParser {
-	///The tag name
-	protected $tag;
+	/**
+	 * The stream context instance
+	 */
+	protected mixed $ctx;
 
 	/**
 	 * Constructor
 	 *
 	 * @param FileLocator $locator The FileLocator instance
 	 * @param PackageInterface $package The Assets Package instance
-	 * @param array $config The config path
+	 * @param string $token The token name
 	 * @param string $tag The tag name
 	 * @param string $output The default output string
-	 * @param array $filters The default filters array
+	 * @param array $filters The default filter array
 	 */
-	public function __construct(FileLocator $locator, PackageInterface $package, array $config, string $tag, string $output, array $filters) {
-		//Save locator
-		$this->locator = $locator;
-
-		//Save assets package
-		$this->package = $package;
-
-		//Set name
-		$this->name = $config['name'];
-
-		//Set scheme
-		$this->scheme = $config['scheme'];
-
-		//Set timeout
-		$this->timeout = $config['timeout'];
-
-		//Set agent
-		$this->agent = $config['agent'];
-
-		//Set redirect
-		$this->redirect = $config['redirect'];
-
-		//Set tag
-		$this->tag = $tag;
-
-		//Set output
-		$this->output = $output;
-
-		//Set filters
-		$this->filters = $filters;
+	public function __construct(protected FileLocator $locator, protected PackageInterface $package, protected string $token, protected string $tag, protected string $output, protected array $filters) {
+		//Set ctx
+		$this->ctx = stream_context_create(
+			[
+				'http' => [
+					#'header' => ['Referer: https://www.openstreetmap.org/'],
+					'max_redirects' => $_ENV['RAPSYSPACK_REDIRECT'] ?? 20,
+					'timeout' => $_ENV['RAPSYSPACK_TIMEOUT'] ?? (($timeout = ini_get('default_socket_timeout')) !== false && $timeout !== "" ? (float)$timeout : 60),
+					'user_agent' => $_ENV['RAPSYSPACK_AGENT'] ?? (($agent = ini_get('user_agent')) !== false && $agent !== "" ? (string)$agent : RapsysPackBundle::getAlias().'/'.RapsysPackBundle::getVersion())
+				]
+			]
+		);
 	}
 
 	/**
@@ -82,14 +80,16 @@ class TokenParser extends AbstractTokenParser {
 	 * @return Node The PackNode
 	 */
 	public function parse(Token $token): Node {
+		//Get parser
 		$parser = $this->parser;
+
+		//Get parser stream
 		$stream = $this->parser->getStream();
 
+		//Set inputs array
 		$inputs = [];
-		$name = $this->name;
-		$output = $this->output;
-		$filters = $this->filters;
 
+		//Set content
 		$content = '';
 
 		//Process the token block until end
@@ -103,19 +103,19 @@ class TokenParser extends AbstractTokenParser {
 				//filter='yui_js'
 				$stream->next();
 				$stream->expect(Token::OPERATOR_TYPE, '=');
-				$filters = array_merge($filters, array_filter(array_map('trim', explode(',', $stream->expect(Token::STRING_TYPE)->getValue()))));
+				$this->filters = array_merge($this->filters, array_filter(array_map('trim', explode(',', $stream->expect(Token::STRING_TYPE)->getValue()))));
 			//The output token
 			} elseif ($stream->test(Token::NAME_TYPE, 'output')) {
 				//output='js/packed/*.js' OR output='js/core.js'
 				$stream->next();
 				$stream->expect(Token::OPERATOR_TYPE, '=');
-				$output = $stream->expect(Token::STRING_TYPE)->getValue();
-			//The name token
-			} elseif ($stream->test(Token::NAME_TYPE, 'name')) {
+				$this->output = $stream->expect(Token::STRING_TYPE)->getValue();
+			//The token name
+			} elseif ($stream->test(Token::NAME_TYPE, 'token')) {
 				//name='core_js'
 				$stream->next();
 				$stream->expect(Token::OPERATOR_TYPE, '=');
-				$name = $stream->expect(Token::STRING_TYPE)->getValue();
+				$this->token = $stream->expect(Token::STRING_TYPE)->getValue();
 			//Unexpected token
 			} else {
 				$token = $stream->getCurrent();
@@ -133,9 +133,9 @@ class TokenParser extends AbstractTokenParser {
 		$stream->expect(Token::BLOCK_END_TYPE);
 
 		//Replace star with sha1
-		if (($pos = strpos($output, '*')) !== false) {
-			//XXX: assetic use substr(sha1(serialize($inputs).serialize($filters).serialize($options)), 0, 7)
-			$output = substr($output, 0, $pos).sha1(serialize($inputs).serialize($filters)).substr($output, $pos + 1);
+		if (($pos = strpos($this->output, '*')) !== false) {
+			//XXX: assetic use substr(sha1(serialize($inputs).serialize($this->filters).serialize($this->output)), 0, 7)
+			$this->output = substr($this->output, 0, $pos).sha1(serialize($inputs).serialize($this->filters)).substr($this->output, $pos + 1);
 		}
 
 		//Process inputs
@@ -143,7 +143,7 @@ class TokenParser extends AbstractTokenParser {
 			//Deal with generic url
 			if (strpos($inputs[$k], '//') === 0) {
 				//Fix url
-				$inputs[$k] = $this->scheme.substr($inputs[$k], 2);
+				$inputs[$k] = ($_ENV['RAPSYSPACK_SCHEME'] ?? 'https').'://'.substr($inputs[$k], 2);
 			//Deal with non url path
 			} elseif (strpos($inputs[$k], '://') === false) {
 				//Check if we have a bundle path
@@ -156,6 +156,7 @@ class TokenParser extends AbstractTokenParser {
 				if (strpos($inputs[$k], '*') !== false || (($a = strpos($inputs[$k], '{')) !== false && ($b = strpos($inputs[$k], ',', $a)) !== false && strpos($inputs[$k], '}', $b) !== false)) {
 					//Get replacement
 					$replacement = glob($inputs[$k], GLOB_NOSORT|GLOB_BRACE);
+
 					//Check that these are working files
 					foreach($replacement as $input) {
 						//Check that it's a file
@@ -163,8 +164,10 @@ class TokenParser extends AbstractTokenParser {
 							throw new Error(sprintf('Input path "%s" from "%s" is not a file', $input, $inputs[$k]), $token->getLine(), $stream->getSourceContext());
 						}
 					}
+
 					//Replace with glob path
 					array_splice($inputs, $k, 1, $replacement);
+
 					//Fix current key
 					$k += count($replacement) - 1;
 				//Check that it's a file
@@ -174,25 +177,15 @@ class TokenParser extends AbstractTokenParser {
 			}
 		}
 
-		//Init context
-		$ctx = stream_context_create(
-			[
-				'http' => [
-					'timeout' => $this->timeout,
-					'user_agent' => $this->agent,
-					'redirect' => $this->redirect,
-				]
-			]
-		);
-
 		//Check inputs
 		if (!empty($inputs)) {
 			//Retrieve files content
 			foreach($inputs as $input) {
 				//Try to retrieve content
-				if (($data = file_get_contents($input, false, $ctx)) === false) {
+				if (($data = file_get_contents($input, false, $this->ctx)) === false) {
 					throw new Error(sprintf('Unable to retrieve input path "%s"', $input), $token->getLine(), $stream->getSourceContext());
 				}
+
 				//Append content
 				$content .= $data;
 			}
@@ -206,22 +199,27 @@ class TokenParser extends AbstractTokenParser {
 		}
 
 		//Check filters
-		if (!empty($filters)) {
+		if (!empty($this->filters)) {
 			//Apply all filters
-			foreach($filters as $filter) {
+			foreach($this->filters as $filter) {
 				//Init args
 				$args = [$stream->getSourceContext(), $token->getLine()];
+
 				//Check if args is available
 				if (!empty($filter['args'])) {
 					//Append args if provided
 					$args += $filter['args'];
 				}
+
 				//Init reflection
 				$reflection = new \ReflectionClass($filter['class']);
+
 				//Set instance args
 				$tool = $reflection->newInstanceArgs($args);
+
 				//Process content
 				$content = $tool->process($content);
+
 				//Remove object
 				unset($tool, $reflection);
 			}
@@ -232,22 +230,22 @@ class TokenParser extends AbstractTokenParser {
 		}
 
 		//Retrieve asset uri
-		//XXX: this path is the merge of services.assets.path_package.arguments[0] and rapsys_pack.output.(css,img,js)
-		if (($outputUrl = $this->package->getUrl($output)) === false) {
-			throw new Error(sprintf('Unable to get url for asset: %s', $output), $token->getLine(), $stream->getSourceContext());
+		//XXX: this path is the merge of services.assets.path_package.arguments[0] and rapsyspack.output.(css,img,js)
+		if (($outputUrl = $this->package->getUrl($this->output)) === false) {
+			throw new Error(sprintf('Unable to get url for asset: %s', $this->output), $token->getLine(), $stream->getSourceContext());
 		}
 
 		//Check if we have a bundle path
-		if ($output[0] == '@') {
+		if ($this->output[0] == '@') {
 			//Resolve it
-			$output = $this->getLocated($output, $token->getLine(), $stream->getSourceContext());
+			$this->output = $this->getLocated($this->output, $token->getLine(), $stream->getSourceContext());
 		}
 
 		//Get filesystem
 		$filesystem = new Filesystem();
 
 		//Create output dir if not present
-		if (!is_dir($dir = dirname($output))) {
+		if (!is_dir($dir = dirname($this->output))) {
 			try {
 				//Create dir
 				//XXX: set as 0775, symfony umask (0022) will reduce rights (0755)
@@ -263,14 +261,14 @@ class TokenParser extends AbstractTokenParser {
 			//Write content to file
 			//XXX: this call is (maybe) atomic
 			//XXX: see https://symfony.com/doc/current/components/filesystem.html#dumpfile
-			$filesystem->dumpFile($output, $content);
+			$filesystem->dumpFile($this->output, $content);
 		} catch (IOExceptionInterface $e) {
 			//Throw error
-			throw new Error(sprintf('Unable to write to: %s', $output), $token->getLine(), $stream->getSourceContext(), $e);
+			throw new Error(sprintf('Unable to write to: %s', $this->output), $token->getLine(), $stream->getSourceContext(), $e);
 		}
 
 		//Set name in context key
-		$ref = new AssignNameExpression($name, $token->getLine());
+		$ref = new AssignNameExpression($this->token, $token->getLine());
 
 		//Set output in context value
 		$value = new TextNode($outputUrl, $token->getLine());
@@ -305,7 +303,7 @@ class TokenParser extends AbstractTokenParser {
 	 * @param Exception $prev The previous exception
 	 * @return string The resolved file path
 	 */
-	public function getLocated(string $file, int $lineno = 0, Source $source = null, \Exception $prev = null): string {
+	public function getLocated(string $file, int $lineno = 0, ?Source $source = null, ?\Exception $prev = null): string {
 		/*TODO: add a @jquery magic feature ?
 		if ($file == '@jquery') {
 			#header('Content-Type: text/plain');
@@ -316,7 +314,7 @@ class TokenParser extends AbstractTokenParser {
 
 		//Check that we have a / separator between bundle name and path
 		if (($pos = strpos($file, '/')) === false) {
-			throw new Error(sprintf('Invalid path "%s"', $file), $token->getLine(), $stream->getSourceContext());
+			throw new Error(sprintf('Invalid path "%s"', $file), $lineno, $source);
 		}
 
 		//Set bundle
@@ -369,7 +367,7 @@ class TokenParser extends AbstractTokenParser {
 				//Catch bundle does not exist or is not enabled exception again
 			} catch(\InvalidArgumentException $e) {
 				//Bail out as bundle or path is invalid and we have no way to know what was meant
-				throw new Error(sprintf('Invalid bundle name "%s" in path "%s". Maybe you meant "%s"', substr($file, 1, $pos - 1), $file, $bundle.'/'.$path), $token->getLine(), $stream->getSourceContext(), $e);
+				throw new Error(sprintf('Invalid bundle name "%s" in path "%s". Maybe you meant "%s"', substr($file, 1, $pos - 1), $file, $bundle.'/'.$path), $lineno, $source, $e);
 			}
 		}